package hathor.giftoftheday;

import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.util.config.Configuration;
import org.bukkit.util.config.ConfigurationNode;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.event.Event;
import org.bukkit.event.Event.Priority;
import org.bukkit.inventory.ItemStack;

public class GiftOfTheDay extends JavaPlugin 
{	
	private static final double sigma = 10; //ecart-type
	private static final String pluginVersion = "V 0.4b";
	
	// Fields	
	private GOTDPlayerListener playerListener = new GOTDPlayerListener(this);
	private int PlayerRoll = -1;
	private String selectedRoll = "";
	
	private static Configuration playersData;
	private static Configuration giftsData;	
	private static  int maxRandom = 0;		
	private static ArrayList<Integer> rollNumbers = new ArrayList<Integer>();
	private static DateFormat dateFormat = new SimpleDateFormat("MM/dd");
	private static Random rand = new Random(System.currentTimeMillis());

	@Override
	public void onEnable() 
	{
		PluginManager manager = getServer().getPluginManager();
		File playersDataFile= new File(getDataFolder() + File.separator + "players.yml");
	    File giftsDataFile= new File(getDataFolder() + File.separator + "gifts.yml");
	    boolean firstPlayerInit = false;
	    boolean firsGifttInit = false;
	    
		manager.registerEvent(Event.Type.PLAYER_JOIN, playerListener, Priority.Normal, this);		
		
		if(!getDataFolder().exists())
		{			
			getDataFolder().mkdir();			
		}
		
		if(!playersDataFile.exists())
		{
			try
			{				
				playersDataFile.createNewFile();
				firstPlayerInit = true;
			}
			catch (IOException ex)
			{
				Logger.getLogger("Minecraft").log(Level.SEVERE, "Error occur while creating players data file");
				ex.printStackTrace();
			}
			catch (Exception ex)
			{
				Logger.getLogger("Minecraft").log(Level.SEVERE, "Unknow error occur");
				ex.printStackTrace();
			}
		}
		
		if(!giftsDataFile.exists())
		{
			try
			{				
				giftsDataFile.createNewFile();
				firsGifttInit = true;
			}
			catch (IOException ex)
			{
				Logger.getLogger("Minecraft").log(Level.SEVERE, "Error occur while creating rewards data file");
				ex.printStackTrace();
			}
			catch (Exception ex)
			{
				Logger.getLogger("Minecraft").log(Level.SEVERE, "Unknow error occur");
				ex.printStackTrace();
			}
		}
		
		playersData = new Configuration(playersDataFile);
		giftsData = new Configuration(giftsDataFile);
		
		if(firstPlayerInit)
		{
			playersData.setHeader("#This file hold players data");
			playersData.setProperty("Players", null);
			playersData.save();
		}
		
		if(firsGifttInit)
		{
			giftsData.setHeader("#This file hold configuration for rewards");
			giftsData.setProperty("Gifts", null);
			giftsData.setProperty("Gifts.100.1.distribution", "linear");			
			giftsData.setProperty("Gifts.100.1.amount", 32);
			giftsData.save();
		}
		
		LoadGiftConfig();
		
		//this.getClass().getResourceAsStream("plugin.yml");
		
		Logger.getLogger("Minecraft").info(String.format("GiftOfTheDay %s loaded",pluginVersion));	
	}

	@Override
	public void onDisable() 
	{
		Logger.getLogger("Minecraft").info(String.format("GiftOfTheDay %s unloaded",pluginVersion));
	}
	
	@Override
	public boolean onCommand(CommandSender sender, Command command,
			String label, String[] args) 
	{
		if(args == null || args.length != 1)
		{
			sender.sendMessage(String.format("Bad command argument"));
			return false;
		}
		Player player = getServer().getPlayer(args[0]);		
		
		if(!sender.hasPermission("gotd.admin"))
		{
			sender.sendMessage("You are not allowed to do this.");
			return true;
		}		
		
		if(player == null)
		{
			sender.sendMessage(String.format("Player %s doesn't exist",args[0]));
			return true;
		}
		
		if(command.getName().equalsIgnoreCase("resetgift"))
		{						
			playersData.load();
			playersData.setProperty(String.format("Players.%s", player.getName()), "00/00");
			playersData.save();
			sender.sendMessage(String.format("Gift of player %s has been reset",player.getName()));
			Logger.getLogger("Minecraft").info(String.format("Gift of player %s has been reset by %s",player.getName(), "someone"));
			return true;
		}
		else if(command.getName().equalsIgnoreCase("regift"))
		{				
			givePlayerGift(player);
			return true;		
		}
		else
			return false;
	}		
	
	/*
	 * Raised when a player join
	 */
	public void onPlayerJoin(Player player)
	{			
		if(player.hasPermission("gotd.receive"))
		{
			String playerName = player.getName();
			if(hasPlayerReceivedGift(playerName))
			{		
				player.sendMessage("You already receive your gift today.");			
			}
			else
			{	
				givePlayerGift(player);			
			}
		}
	}
	
	/*
	 * Load the configuration
	 */
	private void LoadGiftConfig()
	{
		giftsData.load();
		ConfigurationNode rollNode = giftsData.getNode("Gifts");		
		
		for(String key : rollNode.getKeys())
		{			
			Integer i = Integer.parseInt(key);
			maxRandom += i;
			rollNumbers.add(i);
		}
	}
	
	/*
	 * This method determine if a player has already receive his gift or not
	 */
	private boolean hasPlayerReceivedGift(String playerName)
	{	
		boolean returnValue = true;
		playersData.load();
		
		Object o = playersData.getProperty("Players." + playerName);
		if(o == null)
		{	  
			//if player is not yet in the file, we will add him
			 playersData.setProperty("Players." + playerName,dateFormat.format(new Date()));			 
			 returnValue = false;
		}
		else
		{	
			//Just a simple string comparison between the days
			//if the day is different, players has not yet receive his gift			
			String day = o.toString().split("/")[1];
			String todayDay = dateFormat.format(new Date()).split("/")[1];
			if (day.compareTo(todayDay) != 0)
			{				
				 playersData.setProperty("Players." + playerName,dateFormat.format(new Date()));					 
				 returnValue = false;
			}
			else
			{
				returnValue = true;
			}
		}		
		
		playersData.save();
		return returnValue;
	}	
	
	/*
	 * Give a gift to the specified player
	 */
	private void givePlayerGift(Player player)
	{
		Integer itemId = GetItemId(); //get the item
		String path = "Gifts." + selectedRoll + "." + itemId.toString();
		
		Logger.getLogger("Minecraft").info(String.format("Players %s rolls %d (%d max)", player.getName(), PlayerRoll, maxRandom +1));
		
		String distributedMethod = giftsData.getProperty(path + ".distribution").toString();	
		int maxAmount = (Integer) giftsData.getProperty(path + ".amount");
		
		int amount = GetRandomAmount(distributedMethod, maxAmount); //get the amount regarding the specified method
		if(amount <= 0)
		{
			amount = 1;
		}
		
		ItemStack item = new ItemStack(itemId,amount);
		
		if(player.getInventory().firstEmpty() != -1)
		{
			player.getInventory().addItem(item);
		}
		else
		{			
			player.getWorld().dropItem(player.getLocation(), item);
		}		
		player.sendMessage(String.format("You received %d stack(s) of %s for logging in today.", amount, item.getType().name()));
	}	
	
	/*
	 * Get a random item id in the list
	 */
	private int GetItemId()
	{	
		//Load config
		giftsData.load();
		ConfigurationNode rollNode = giftsData.getNode("Gifts");		
		
		int diceRoll = rand.nextInt(maxRandom +1); // <- the "famous" roll :p
		PlayerRoll = diceRoll; // <- just to hold the number outside the method in order to log
		
		int i = 0;
		//look for the nearest roll number in the list
		while(diceRoll < rollNumbers.get(i))
		{
			i++;
		}		
				
		selectedRoll = rollNumbers.get(i).toString();
		rollNode = giftsData.getNode("Gifts."+ selectedRoll); //get the node
		
		List<String> keys = rollNode.getKeys(); 
		
		return Integer.parseInt(keys.get(rand.nextInt(keys.size()))); //choose and return an item		
	}
	
	/*
	 * Return a random amount depending of the specified method
	 * Args:
	 * 	distributedMethod = the method used for distribution. (Default is a simple linear distribution)
	 *  maxAmount = Maximum amount a player can receive.
	 */
	private int GetRandomAmount(String distributedMethod, Integer maxAmount)
	{
		if(distributedMethod == "normal")
		{			
			return (int)(maxAmount + Math.round(sigma * Math.sqrt(-2 * Math.log(rand.nextDouble())) * Math.cos(2 * Math.PI * rand.nextDouble())));
		}
		else if (distributedMethod == "exponential")
		{
			return (int) (maxAmount * Math.exp(rand.nextDouble()));
		}
		else if (distributedMethod == "invertexponential")
		{
			return (int) (-maxAmount * Math.log(rand.nextDouble()));
		}
		else
		{
			return  rand.nextInt(maxAmount);
		}
	}	
}
